package mvd.jester;

import java.util.HashSet;
import java.util.Set;
import mvd.jester.info.SchedulingInfo;
import mvd.jester.priority.PriorityManager;

/**
 * PriorityPair
 */
public class ResultCollector<T extends TypeInterface>
        implements Comparable<ResultCollector<? extends TypeInterface>> {
    private final PriorityManager priorityManager;
    private final T abstractValue;
    private final Set<SchedulingInfo> schedulingInfos;

    public ResultCollector(PriorityManager priorityManager, T abstractValue) {
        this.abstractValue = abstractValue;
        this.priorityManager = priorityManager;
        this.schedulingInfos = new HashSet<>();
    }

    public ResultCollector(ResultCollector<T> resultCollector) {
        this.abstractValue = resultCollector.abstractValue;
        this.priorityManager = resultCollector.priorityManager;
        this.schedulingInfos = new HashSet<>();
    }

    /**
     * @return the priorityManager
     */
    public PriorityManager getPriorityManager() {
        return priorityManager;
    }

    /**
     * @return the abstractValue
     */
    public T getAbstractValue() {
        return abstractValue;
    }

    public String getName() {
        return abstractValue.getName() + "_" + priorityManager.getName();
    }


    public boolean addResult(SchedulingInfo schedulingInfo) {
        return this.schedulingInfos.add(schedulingInfo);
    }

    public Set<SchedulingInfo> getResults() {
        return schedulingInfos;
    }

    @Override
    public int compareTo(ResultCollector<? extends TypeInterface> o) {
        return getName().compareTo(o.getName());
    }
}
