package mvd.jester.model;

import java.lang.reflect.Type;
import java.util.TreeSet;
import com.google.gson.JsonArray;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonParseException;
import mvd.jester.priority.PriorityManager;
import mvd.jester.priority.RateMonotonic;

/**
 * SortedTaskSet
 */
public class SortedTaskSet<T extends Task> extends TreeSet<T> {

    private static final long serialVersionUID = 4808544133562675597L;

    public SortedTaskSet(final PriorityManager priorityMananger) {
        super((t1, t2) -> priorityMananger.compare(t1, t2));
    }


    public double getUtilization() {
        return super.stream().mapToDouble(T::getUtilization).sum();
    }


    public double getParallelTaskRatio() {
        final long parallelTasks =
                super.stream().filter(t -> t.getMaximumParallelism() > 1).count();

        return (double) parallelTasks / super.size();
    }

    public static class Deserializer<T extends Task> implements JsonDeserializer<SortedTaskSet<T>> {

        @Override
        public SortedTaskSet<T> deserialize(final JsonElement json, final Type typeOfT,
                final JsonDeserializationContext context) throws JsonParseException {
            final SortedTaskSet<T> taskSet = new SortedTaskSet<>(new RateMonotonic());
            if (json.isJsonArray()) {
                final JsonArray array = json.getAsJsonArray();
                array.forEach(e -> {
                    taskSet.add(context.deserialize(e, SynchronousTask.class));
                });
            }
            return taskSet;
        }
    }
}
