package mvd.jester.tests;

import java.math.RoundingMode;
import java.util.Map;
import com.google.common.math.LongMath;
import mvd.jester.info.TerminationInfo;
import mvd.jester.model.DagTask;
import mvd.jester.model.Segment;
import mvd.jester.model.Task;

public interface TypeFunction {

    public double getTaskInterference(final DagTask task,
                final Map<Task, TerminationInfo> responseTimes,
                final long interval,
            final long parallelism);

    public String getType();

    public class UnkownStructure implements TypeFunction {
    
        @Override
        public double getTaskInterference(final DagTask task,
                final Map<Task, TerminationInfo> responseTimes,
                final long interval,
                final long parallelism) {
            if (responseTimes.containsKey(task)) {
                final long responseTime = responseTimes.get(task).getResponseTime();
                final long minWcet = task.getCriticalPath();
                final long period = task.getPeriod();
                final long amountOfJobs =
                        (LongMath.divide(interval + responseTime - minWcet, period, RoundingMode.FLOOR)
                                + 1);

                double workload = 0;

                for (final Segment s : task.getWorkloadDistribution()) {
                    final long numberOfThreads =
                             task.getNumberOfThreads();
                    if (numberOfThreads >= parallelism) {
                        workload += (double)(s.getNumberOfJobs() * s.getJobWcet()) / numberOfThreads;
                    }
                }

                final double interference = amountOfJobs * workload;

                return interference;
            } else {
                throw new RuntimeException("Task was not found in task set!");
            }
        }

        @Override
        public String getType() {
            return "SU";
        }
    }

    public class KownStructure implements TypeFunction {
    
        @Override
        public double getTaskInterference(final DagTask task,
                final Map<Task, TerminationInfo> responseTimes,
                final long interval,
                final long parallelism) {
            if (responseTimes.containsKey(task)) {
                final long responseTime = responseTimes.get(task).getResponseTime();
                final long minWcet = task.getCriticalPath();
                final long period = task.getPeriod();
                final long amountOfJobs =
                        (LongMath.divide(interval + responseTime - minWcet, period, RoundingMode.FLOOR)
                                + 1);

                double workload = 0;

                for (final Segment s : task.getWorkloadDistribution()) {
                    final long numberOfThreads =
                            s.getNumberOfJobs() > 1 ? task.getNumberOfThreads() : 1;
                    if (numberOfThreads >= parallelism) {
                        workload += (double)(s.getNumberOfJobs() * s.getJobWcet()) / numberOfThreads;
                    }
                }

                final double interference = amountOfJobs * workload;

                return interference;
            } else {
                throw new RuntimeException("Task was not found in task set!");
            }
        }

        @Override
        public String getType() {
            return "SK";
        }
    }
    
}